import numpy
import random

def blank_frame(width, height):
    frame = numpy.zeros((height, width, 3), dtype=numpy.uint8)
    return frame

def moving_rectangles():
    return

def moving_points():
    return

def blinking_rectangles():
    return

def blinking_points(regions, hot_spots=False):
    return
